SystemOrganization addCategory: #Loom!

TestCase subclass: #LoomTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

LoomTest subclass: #LoomAcquiringProcessTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomAcquiringProcessTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/13/2012 08:44'!
testSimulation

	| pool t1 t2 t3 |
	pool := LoomSimulatedSchedulingStrategy new.
	pool add: (t1 := [] fork).
	pool add: (t2 := [] fork).
	pool add: (t3 := [] fork).
	
	self assert: pool getNextProcessToRun == t1.
	self assert: pool getNextProcessToRun == t2.
	self assert: pool getNextProcessToRun == t3.
	
	t1 terminate.
	t2 terminate.
	t3 terminate.! !

LoomTest subclass: #LoomAcquiringThreadStrategyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

LoomTest subclass: #LoomEventTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomEventTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/13/2012 21:01'!
testSampledEvent
	| t profiler |
	profiler := LoomProfiler new.
	t := 0.
	profiler on: LoomSamplingEvent do: [ :event | t := t + 1 ].
	
	self assert: t = 0.
	profiler sampleExecution.
	self assert: t = 1! !

!LoomEventTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/11/2012 22:48'!
testSequence

	| event1 event2 |
	event1 := LoomSampledEvent new.
	(Delay forMilliseconds: 2) wait.
	event2 := LoomSampledEvent new.
	
	self assert: event1 time < event2 time! !

LoomTest subclass: #LoomPlatformTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomPlatformTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/6/2012 09:53'!
testLoomTime

	| time |
	time := LoomTime currentTimeAsMilliseconds.
	(Delay forMilliseconds: 2) wait.
	self assert: (LoomTime currentTimeAsMilliseconds - time ) >= 2! !

!LoomPlatformTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/6/2012 10:00'!
testProcessor

	"Just to make sure we have what we expect"
	self assert: (Processor class == ProcessorScheduler)! !

LoomTest subclass: #LoomProfilerTest
	instanceVariableNames: 'thread1 thread2 profiler thread3'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomProfilerTest methodsFor: 'util' stamp: 'AlexandreBergel 9/5/2012 14:23'!
isScenarioOver
	(Array with: thread1 with: thread2 with: thread3) 
		do: [ :t | (t notNil and: [ t isTerminated not ]) ifTrue: [ ^ false ] ].
	^ true
! !

!LoomProfilerTest methodsFor: 'scenarios' stamp: 'AlexandreBergel 9/12/2012 14:46'!
runThread1
	thread1 := [ 200 timesRepeat: [ self wait ] ] fork! !

!LoomProfilerTest methodsFor: 'scenarios' stamp: 'AlexandreBergel 9/12/2012 14:46'!
runThread2
	thread2 := [ 100 timesRepeat: [ self wait2 ] ] fork! !

!LoomProfilerTest methodsFor: 'scenarios' stamp: 'AlexandreBergel 9/12/2012 08:38'!
runThread3
	thread3 := [ 50 timesRepeat: [ self wait3 ] ] fork! !

!LoomProfilerTest methodsFor: 'scenarios' stamp: 'AlexandreBergel 9/12/2012 08:36'!
scenario1
	self runThread1.
	self runThread2! !

!LoomProfilerTest methodsFor: 'scenarios' stamp: 'AlexandreBergel 9/12/2012 08:38'!
scenario2
	self runThread1.
	self runThread2.
	self runThread3! !

!LoomProfilerTest methodsFor: 'running' stamp: 'AlexandreBergel 9/5/2012 10:51'!
setUp
	super setUp.
	profiler := LoomProfiler new.! !

!LoomProfilerTest methodsFor: 'running' stamp: 'AlexandreBergel 9/12/2012 15:00'!
tearDown
	profiler stopProfiling.
	self waitForScenarioEnd! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/12/2012 11:55'!
testBasic

	self scenario1.
	profiler profile: (Array with: thread1 with: thread2).
	self waitForScenarioEnd.
	profiler stopProfiling.
	
	self assert: profiler profilingTime > 10.
	self assert: profiler numberOfSamplings > 1.
	
	"Thread1 consume twice more the CPU than Thead2. Actually, thread2 is sleeping twice much."

	"self assert: (profiler consumptionTimeOf: thread1) > (profiler consumptionTimeOf: thread2)."
	self assert: (profiler consumptionTimeOf: thread1) >= 0.2.
	self assert: (profiler consumptionTimeOf: thread2) <= 0.7.! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/12/2012 11:14'!
testGetLoomThreadFrom
	| p ps |
	p := [ ] fork.

	profiler profile: (Array with: p).
	self assert: (profiler getThreadFrom: p) process == p.
	self assert: (profiler getThreadFrom: p) id = 1.
	self assert: (profiler getThreadFrom: p) profiler == profiler! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/12/2012 09:23'!
testGetLoomThreads
	"
	self debug: #testGetLoomThreads
	"
	| p ps |
	p := [ ] fork.

	ps := profiler getLoomThreadsFrom: (Array with: p).

	self assert: ps size = 1.
	self assert: ps first process == p.
	self assert: ps first id = 1.

	self assert: ps first profiler == profiler! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/12/2012 14:50'!
testInitialization

	self assert: profiler numberOfThreads = 0! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/5/2012 11:40'!
testIsProfiling

	self deny: profiler isProfiling.
	profiler startProfiling.
	self assert: profiler isProfiling.
	
	profiler stopProfiling.
	self deny: profiler isProfiling.
	
! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/6/2012 09:03'!
testPrintingThreads

	self shouldnt: [ profiler allCurrentThreads do: [ :t | t printString ] ] raise: Error! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/6/2012 08:42'!
testPriority
	| p ps thread |
	p := [ ] fork.
	profiler profile: (Array with: p).
	thread := profiler getThreadFrom: p.

	self assert: thread priority = p priority! !

!LoomProfilerTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/12/2012 14:42'!
testThreadCreation

	| p ps t1 t2 |
	p := [ ] fork.

	profiler profile: (Array with: p).
	t1 := profiler getThreadFrom: p.
	t2 := profiler getThreadFrom: p.

	self assert: t1 process == t2 process.
	self assert: t1 == t2
	
! !

!LoomProfilerTest methodsFor: 'thread state' stamp: 'AlexandreBergel 9/6/2012 08:45'!
testThreadState

	| loomThread |
	self scenario1.
	profiler profile: (Array with: thread1).
	loomThread := profiler getThreadFrom: thread1. 
	
	self deny: thread1 isTerminated.
	self deny: loomThread isTerminated.
	
	self waitForScenarioEnd.
	
	self assert: thread1 isTerminated.
	self assert: loomThread isTerminated.
! !

!LoomProfilerTest methodsFor: 'util' stamp: 'AlexandreBergel 9/5/2012 09:55'!
wait
	(Delay forMilliseconds: 2) wait! !

!LoomProfilerTest methodsFor: 'util' stamp: 'AlexandreBergel 9/5/2012 09:55'!
wait2
	(Delay forMilliseconds: 4) wait! !

!LoomProfilerTest methodsFor: 'util' stamp: 'AlexandreBergel 9/5/2012 14:25'!
wait3
	(Delay forMilliseconds: 8) wait! !

!LoomProfilerTest methodsFor: 'util' stamp: 'AlexandreBergel 9/5/2012 10:12'!
waitForScenarioEnd
	[ self isScenarioOver not ] whileTrue: [ self wait ]! !

LoomTest subclass: #LoomSimulatedChedulingStrategyTest
	instanceVariableNames: 'profiler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomSimulatedChedulingStrategyTest methodsFor: 'running' stamp: 'AlexandreBergel 9/13/2012 08:42'!
setUp
	super setUp.
	profiler := LoomProfiler new.! !

!LoomSimulatedChedulingStrategyTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/13/2012 08:59'!
testScenario1

	| schedulingStrategy |
	schedulingStrategy := LoomSimulatedSchedulingStrategy scenario1.
	profiler schedulingStrategy: schedulingStrategy.
	profiler acquiringThreadStrategy: 
				(LoomSelectiveStrategy forProcesses: schedulingStrategy processes profile: profiler).
				
	profiler sampleExecutionsUntilReady.
	
	(profiler allCurrentThreads collect: #consumptionTime) do: [ :v |
		self assert: (v - 0.33) abs < 0.03 ]! !

!LoomSimulatedChedulingStrategyTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/13/2012 09:01'!
testScenario2

	| schedulingStrategy |
	schedulingStrategy := LoomSimulatedSchedulingStrategy scenario2.
	profiler schedulingStrategy: schedulingStrategy.
	profiler acquiringThreadStrategy: 
				(LoomSelectiveStrategy forProcesses: schedulingStrategy processes profile: profiler).
				
	profiler sampleExecutionsUntilReady.
	
	self assert: (profiler allCurrentThreads collect: #consumptionTime) asSet = #(0.71 0.14 0.14) asSet! !

!LoomSimulatedChedulingStrategyTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/13/2012 08:44'!
testThreadSchedulingSimulation

	| t1 t2 pool |
	t1 := Process new priority: 40; yourself.
	t2 := Process new priority: 40; yourself.
	
	pool := LoomSimulatedSchedulingStrategy new.
	pool 
		add: t1; add: t2; add: t1;
		add: t2; add: t1; add: t2.
	
	profiler acquiringThreadStrategy: 
				(LoomSelectiveStrategy forProcesses: (Array with: t1 with: t2) profile: profiler).
	
	profiler schedulingStrategy: pool.

	profiler sampleExecution; sampleExecution; sampleExecution.	
	self assert: (profiler getThreadFrom: t1) tick = 2.
	self assert: (profiler getThreadFrom: t2) tick = 1.
	self assert: profiler numberOfSamplings = 3.

	self assert: (profiler consumptionTimeOf: t1) = (2/3) asFloat.
	self assert: (profiler consumptionTimeOf: t2) = (1/3) asFloat.
	
	profiler resetTicks.
	profiler sampleExecution; sampleExecution; sampleExecution.	
		
	self assert: (profiler consumptionTimeOf: t2) = (2/3) asFloat.
	self assert: (profiler consumptionTimeOf: t1) = (1/3) asFloat.
	
	self assert: profiler numberOfSamplings < profiler numberOfSamplingsBetweenReset! !

!LoomSimulatedChedulingStrategyTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/13/2012 08:44'!
testThreadSchedulingSimulation2

	| t1 t2 pool |
	t1 := Process new priority: 40; yourself.
	t2 := Process new priority: 40; yourself.

	"We fill the pool with twice much processes than the amount of sample between two resets of ticks"
	pool := LoomSimulatedSchedulingStrategy new.
	profiler numberOfSamplingsBetweenReset timesRepeat: [ pool add: t1; add: t2 ].
	
	"We set the profiler with a selective strategy, which is necessary "
	profiler acquiringThreadStrategy: 
				(LoomSelectiveStrategy forProcesses: (Array with: t1 with: t2) profile: profiler).
	profiler schedulingStrategy: pool.

	(profiler numberOfSamplingsBetweenReset - 1) timesRepeat: [ profiler sampleExecution ].
	self assert: profiler numberOfSamplings = (profiler numberOfSamplingsBetweenReset - 1).
	
	self assert: (profiler getThreadFrom: t1) tick = (profiler numberOfSamplingsBetweenReset / 2).
	self assert: (profiler getThreadFrom: t2) tick = (profiler numberOfSamplingsBetweenReset / 2- 1).

	profiler sampleExecution. 
	self assert: profiler numberOfSamplings = 0.
	self assert: (profiler getThreadFrom: t1) tick = 0.
	self assert: (profiler getThreadFrom: t2) tick = 0.
! !

LoomTest subclass: #LoomThreadTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomThreadTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/13/2012 11:22'!
testNamedProcess
	| p thread |
	p := Process new.
	p name: 'My little thread'.

	thread := LoomThread new.
	thread process: p.
	
	self assert: thread printString = ('[0%] (1) T  My little thread nil ', p hash printString).
	
! !

!LoomThreadTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/13/2012 11:17'!
testPriority

	self assert: LoomThread new priority = 1.
	self assert: (LoomThread new process: Process new) priority = 1! !

!LoomThreadTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/13/2012 11:18'!
testUnamedProcess
	| p thread |
	p := Process new.
	
	thread := LoomThread new.
	thread process: p.
	
	self assert: thread printString = ('[0%] (1) T nil ', p hash printString).
	
! !

!LoomThreadTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/13/2012 11:16'!
testWronglyInitialized

	self shouldnt: [ LoomThread new printString ] raise: Error.
	self assert:  LoomThread new printString = LoomThread wronglyInitializedThreadMessage.
! !

Announcement subclass: #LoomEvent
	instanceVariableNames: 'time'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomEvent class methodsFor: 'instance creation' stamp: 'AlexandreBergel 9/11/2012 22:43'!
new
	^ self basicNew initialize! !

!LoomEvent methodsFor: 'initialize-release' stamp: 'AlexandreBergel 9/11/2012 22:44'!
initialize
	time := LoomTime currentTimeAsMilliseconds! !

!LoomEvent methodsFor: 'accessing' stamp: 'AlexandreBergel 9/11/2012 22:45'!
time
	^ time! !

LoomEvent subclass: #LoomMeasurementReadyEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

LoomEvent subclass: #LoomSampledEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

LoomEvent subclass: #LoomSamplingEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

Object subclass: #LoomObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

LoomObject subclass: #LoomAcquiringProcessStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomAcquiringProcessStrategy commentStamp: 'AlexandreBergel 9/12/2012 15:33' prior: 0!
LoomAcquiringProcessStrategy is a generic way to obtain Smalltalk process from the system. They may either be user selected, or programmatically selected.!

!LoomAcquiringProcessStrategy methodsFor: 'hooks' stamp: 'AlexandreBergel 9/12/2012 15:37'!
getNextProcessToRun
	"Return the next Smalltalk process that will be run by the system scheduler"

	self subclassResponsibility ! !

LoomAcquiringProcessStrategy subclass: #LoomSimulatedSchedulingStrategy
	instanceVariableNames: 'processesToDispatch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomSimulatedSchedulingStrategy class methodsFor: 'scenarios' stamp: 'AlexandreBergel 9/13/2012 10:03'!
scenario1
	"Three processes. t1 33%, t2 33%, t3 33%"
	
	| t1 t2 t3 nbSamplingBetweenReset pt1 pt2 pt3 pool |
	t1 := Process new priority: 41; yourself.
	t2 := Process new priority: 42; yourself.
	t3 := Process new priority: 43; yourself.
	
	nbSamplingBetweenReset := 50.

	pt1 := 0.33.
	pt2 := 0.33.
	pt3 := 0.33.
	
	pool := self new.

	5 timesRepeat: 
		[ (pt1 * nbSamplingBetweenReset) asInteger timesRepeat: [ pool add: t1 ].
		  (pt2 * nbSamplingBetweenReset) asInteger timesRepeat: [ pool add: t2 ].
		  (pt3 * nbSamplingBetweenReset) asInteger timesRepeat: [ pool add: t3 ] ].	

	^ pool! !

!LoomSimulatedSchedulingStrategy class methodsFor: 'scenarios' stamp: 'AlexandreBergel 9/13/2012 10:03'!
scenario2
	"Three processes. t1 70%, t2 15%, t3 15%"
	
	| t1 t2 t3 nbSamplingBetweenReset pt1 pt2 pt3 pool |
	t1 := Process new priority: 41; yourself.
	t2 := Process new priority: 42; yourself.
	t3 := Process new priority: 43; yourself.
	
	nbSamplingBetweenReset := 50.

	pt1 := 0.70.
	pt2 := 0.15.
	pt3 := 0.15.
	
	pool := self new.

	5 timesRepeat: 
		[ (pt1 * nbSamplingBetweenReset) asInteger timesRepeat: [ pool add: t1 ].
		  (pt2 * nbSamplingBetweenReset) asInteger timesRepeat: [ pool add: t2 ].
		  (pt3 * nbSamplingBetweenReset) asInteger timesRepeat: [ pool add: t3 ] ].	

	^ pool! !

!LoomSimulatedSchedulingStrategy methodsFor: 'adding' stamp: 'AlexandreBergel 9/12/2012 18:11'!
add: process
	"Add a process to the strategy"
	
	processesToDispatch add: process! !

!LoomSimulatedSchedulingStrategy methodsFor: 'hooks' stamp: 'AlexandreBergel 9/12/2012 15:40'!
getNextProcessToRun
	^ processesToDispatch removeFirst! !

!LoomSimulatedSchedulingStrategy methodsFor: 'initialize-release' stamp: 'AlexandreBergel 9/12/2012 15:39'!
initialize 
	super initialize.
	processesToDispatch := OrderedCollection new! !

!LoomSimulatedSchedulingStrategy methodsFor: 'accessing' stamp: 'AlexandreBergel 9/13/2012 08:48'!
processes
	"Return the list of processes we want to monitor"
	
	^ processesToDispatch asSet asArray! !

LoomAcquiringProcessStrategy subclass: #LoomSmalltalkSchedulingStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomSmalltalkSchedulingStrategy methodsFor: 'hooks' stamp: 'AlexandreBergel 9/12/2012 15:38'!
getNextProcessToRun
	^ Processor nextReadyProcess! !

LoomObject subclass: #LoomAcquiringThreadStrategy
	instanceVariableNames: 'profiledThreads profiler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomAcquiringThreadStrategy commentStamp: 'AlexandreBergel 9/12/2012 15:33' prior: 0!
LoomAcquiringThreadStrategy is a generic way to obtain loom threads from the system. They may either be user selected, or programmatically selected.!

!LoomAcquiringThreadStrategy class methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 11:31'!
forProfiler: aProfiler
	^ super new profiler: aProfiler; yourself! !

!LoomAcquiringThreadStrategy class methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 11:31'!
new
	self error: 'Should not be used'! !

!LoomAcquiringThreadStrategy methodsFor: 'util' stamp: 'AlexandreBergel 9/12/2012 19:46'!
getLoomThreadsFrom: processes
	"Return a collection of LoomThreads from a list of Smalltalk processes"

	| id loomThreads newProcesses |
	id := 0.
	loomThreads := profiledThreads select: [ :loomThread | 
							id := id max: loomThread id.
							processes includes: loomThread process ].
	
	"processes that were not profiled before (i.e., have no loom thread associated to them)"
	newProcesses := processes asOrderedCollection.
	newProcesses removeAll: (loomThreads collect: #process). 
	
	loomThreads addAll: 
		(newProcesses collect: [ :p |
			id := id + 1.
			LoomThread new
				process: p;
				id: id;
				profiler: profiler;
				yourself ]).

	^ profiledThreads := loomThreads! !

!LoomAcquiringThreadStrategy methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 09:13'!
getThreadFrom: aSmalltalkProcess
	"Return the loom thread associated to the smalltalk process"
	
	^ self profiledThreads detect: [ :t | t process == aSmalltalkProcess ]! !

!LoomAcquiringThreadStrategy methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 14:40'!
getThreadFrom: aSmalltalkProcess ifAbsent: aBlock
	"Return the loom thread associated to the smalltalk process"

	"Not sure whether this method is useful at all"
	^ self profiledThreads detect: [ :t | t process == aSmalltalkProcess ] ifNone: aBlock! !

!LoomAcquiringThreadStrategy methodsFor: 'initialize-release' stamp: 'AlexandreBergel 9/12/2012 13:09'!
initialize

	profiledThreads := OrderedCollection new! !

!LoomAcquiringThreadStrategy methodsFor: 'accessing' stamp: 'AlexandreBergel 9/12/2012 09:04'!
numberOfThreads
	"Return the number of threads profiled"

	^ profiledThreads size! !

!LoomAcquiringThreadStrategy methodsFor: 'hooks' stamp: 'AlexandreBergel 9/12/2012 08:55'!
processesToProfile
	"Return the list of Smalltalk processes to profile"

	self subclassResponsibility ! !

!LoomAcquiringThreadStrategy methodsFor: 'accessing-computed' stamp: 'AlexandreBergel 9/12/2012 09:11'!
profiledThreads 
	"Return the list of loom processes for the profiled Smalltalk processes"
	
	^ self getLoomThreadsFrom: self processesToProfile! !

!LoomAcquiringThreadStrategy methodsFor: 'accessing' stamp: 'AlexandreBergel 9/12/2012 11:22'!
profiler: aProfiler
	profiler := aProfiler ! !

LoomAcquiringThreadStrategy subclass: #LoomAllThreadsStrategy
	instanceVariableNames: 'processesToProfile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomAllThreadsStrategy methodsFor: 'hooks' stamp: 'AlexandreBergel 9/12/2012 08:57'!
processesToProfile
 	"Maybe a weak collection would work better, no idea so far. Let's wait for a user to complain :-)"
	
	^" WeakSet withAll:"
		(Process allInstances
				reject: [ :each | each isTerminated ]).! !

LoomAcquiringThreadStrategy subclass: #LoomSelectiveStrategy
	instanceVariableNames: 'processesToProfile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomSelectiveStrategy class methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 18:20'!
forProcesses: processCollection profile: profiler

	^ (super forProfiler: profiler) processesToProfile: processCollection; yourself! !

!LoomSelectiveStrategy class methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 09:01'!
new
	self error: 'should not be used, use for: instead'! !

!LoomSelectiveStrategy methodsFor: 'hooks' stamp: 'AlexandreBergel 9/12/2012 08:55'!
processesToProfile
	"Smalltalk processes"
	
	^ processesToProfile! !

!LoomSelectiveStrategy methodsFor: 'accessing' stamp: 'AlexandreBergel 9/12/2012 08:52'!
processesToProfile: processes
	"Smalltalk processes as input"
	
	processesToProfile := processes! !

!LoomObject class methodsFor: 'instance creation' stamp: 'AlexandreBergel 9/6/2012 09:34'!
new
	^ self basicNew initialize! !

!LoomObject methodsFor: 'initialize-release' stamp: 'AlexandreBergel 9/6/2012 09:34'!
initialize! !

LoomObject subclass: #LoomProfiler
	instanceVariableNames: 'beginingProfilingTime profilingTime profilingProcess numberOfSamplings announcer acquiringThreadStrategy schedulingStrategy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 18:20'!
acquiringThreadStrategy: strategy

	acquiringThreadStrategy := strategy! !

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/13/2012 08:55'!
allCurrentThreads
	"Return a weak array of loom threads"

	^ self getLoomThreadsFrom: self allMonitoredProcesses! !

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/13/2012 08:55'!
allMonitoredProcesses
	"Return the list of processes to monitor"
	
	^ acquiringThreadStrategy processesToProfile! !

!LoomProfiler methodsFor: 'events' stamp: 'AlexandreBergel 9/11/2012 22:04'!
announce: anEvent

	self announcer announce: anEvent! !

!LoomProfiler methodsFor: 'accessing' stamp: 'AlexandreBergel 9/11/2012 21:44'!
announcer
	^ announcer ifNil: [ announcer := Announcer new ]! !

!LoomProfiler methodsFor: 'accessing computed' stamp: 'AlexandreBergel 9/5/2012 11:58'!
consumptionTimeOf: aSmalltalkProcess
	^ ((self getThreadFrom: aSmalltalkProcess) tick / numberOfSamplings) asFloat! !

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/13/2012 21:27'!
createProfilingThread
	profilingProcess := [ [ self isProfiling ] whileTrue: [ 
							self sampleExecution.
							(Delay forMilliseconds: 10) wait.
						 ] ] forkAt: 50.
! !

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/12/2012 09:25'!
getLoomThreadsFrom: processes
	"Return a collection of LoomThreads from a list of Smalltalk processes"

	"Maybe this method can be removed on some point...."
	| loomThreads |
	loomThreads := acquiringThreadStrategy getLoomThreadsFrom: processes.
	loomThreads do: [ :loomThread | loomThread profiler: self ].
	^ loomThreads! !

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/13/2012 08:52'!
getNextProcessToRun
	^ schedulingStrategy getNextProcessToRun
! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 09:02'!
getThreadFrom: aSmalltalkProcess
	"Return the loom thread associated to the smalltalk process"
	
	^ acquiringThreadStrategy getThreadFrom: aSmalltalkProcess! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 09:03'!
getThreadFrom: aSmalltalkProcess ifAbsent: aBlock
	"Return the loom thread associated to the smalltalk process"
	
	^ acquiringThreadStrategy getThreadFrom: aSmalltalkProcess ifAbsent: aBlock! !

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/12/2012 18:42'!
increaseTick: nextProcessToRun
	| loomThread |
	loomThread := self getThreadFrom: nextProcessToRun ifAbsent: [ ^ self ].
	loomThread increaseTick.
	numberOfSamplings := numberOfSamplings + 1.
	
	(numberOfSamplings = self numberOfSamplingsBetweenReset) 
		ifTrue: [ self resetTicks ]! !

!LoomProfiler methodsFor: 'initialize-release' stamp: 'AlexandreBergel 9/12/2012 18:29'!
initialize
	super initialize.
	profilingTime := 0.
	beginingProfilingTime := 0.
	numberOfSamplings := 0.
	acquiringThreadStrategy := LoomAllThreadsStrategy forProfiler: self.
	schedulingStrategy := LoomSmalltalkSchedulingStrategy new.! !

!LoomProfiler methodsFor: 'testing' stamp: 'AlexandreBergel 9/5/2012 11:41'!
isProfiling
	"Are we profiling?"
	
	^ beginingProfilingTime ~= 0! !

!LoomProfiler methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 19:32'!
numberOfSamplings
	^ numberOfSamplings! !

!LoomProfiler methodsFor: 'accessing' stamp: 'AlexandreBergel 9/12/2012 19:28'!
numberOfSamplingsBetweenReset
	"Arbitrary number"
	"Some experiements are necessary to find the ideal number"

	^ 50! !

!LoomProfiler methodsFor: 'accessing computed' stamp: 'AlexandreBergel 9/12/2012 09:04'!
numberOfThreads
	"Return the number of threads profiled"
	
	^ acquiringThreadStrategy numberOfThreads! !

!LoomProfiler methodsFor: 'events' stamp: 'AlexandreBergel 9/11/2012 22:35'!
on: anEvent do: aBlock
	"Register the callback one arg aBlock for the event anEvent, a subclass of LoomEvent"
	
	self announcer when: anEvent do: aBlock! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 11:37'!
profile: processCollection
	"profile a collection of Smalltalk processes"
	
	acquiringThreadStrategy := LoomSelectiveStrategy forProcesses: processCollection profile: self.
	self startProfiling! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 18:19'!
profileAllProcesses
	"profile all the active processes"
	
	self acquiringThreadStrategy: (LoomAllThreadsStrategy forProfiler: self).
	self startProfiling! !

!LoomProfiler methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 10:36'!
profilingTime
	^ profilingTime! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/13/2012 09:34'!
resetTicks
	"Set to 0 the ticks of the threads and the number of sampling of the profiler"
	"This is important to regularly do this"
	
	self announce: LoomMeasurementReadyEvent.
	
	self threadsDo: #resetTick.
	numberOfSamplings := 0.
	
"Transcript show: 'reset!!' ; cr."! !

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/12/2012 18:41'!
sampleExecution
	"This method is regularly invoved by the profiler. It is where all happens!!"
	"Simply increases the tick of the loom thread next to be executed"

	| loomThread nextProcessToRun |
	nextProcessToRun := self getNextProcessToRun.
	self announce: LoomSamplingEvent.
	nextProcessToRun isNil
		ifTrue: [ ^ self ].
	
	self increaseTick: nextProcessToRun.
	self announce: LoomSampledEvent! !

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/13/2012 08:59'!
sampleExecutionsUntilReady
	"This method is useful in the test"

	self numberOfSamplingsBetweenReset - 1 timesRepeat: [ self sampleExecution ]! !

!LoomProfiler methodsFor: 'accessing' stamp: 'AlexandreBergel 9/13/2012 08:52'!
schedulingStrategy: aStrategy
	"Useful to set a new scheduling strategy. Interesting when doing test"
	
	schedulingStrategy := aStrategy! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/11/2012 21:42'!
startProfiling
	"Start the monitoring"
	
	self startTimer.
	self createProfilingThread! !

!LoomProfiler methodsFor: 'private' stamp: 'AlexandreBergel 9/6/2012 09:50'!
startTimer
	beginingProfilingTime := LoomTime currentTimeAsMilliseconds! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/11/2012 21:42'!
stopProfiling
	"Stop the monitoring"
	
	profilingTime := LoomTime currentTimeAsMilliseconds - beginingProfilingTime.
	beginingProfilingTime := 0! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 09:02'!
threadsDo: aBlock
	"Evaluate a block for each loom thread"
	
	acquiringThreadStrategy profiledThreads do: aBlock! !

!LoomProfiler methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 15:16'!
threadsDo: aBlock using: sortingStrategy
	"Evaluate a block for each loom thread"
	
	(sortingStrategy sort: acquiringThreadStrategy profiledThreads) do: aBlock! !

LoomObject subclass: #LoomSimulation
	instanceVariableNames: 'processes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomSimulation commentStamp: 'AlexandreBergel 9/13/2012 09:06' prior: 0!
LoomSimulation stimulate CPU activity by creating threads 

Instance Variables:
	processes	List of processes!

!LoomSimulation methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 9/13/2012 09:07'!
initialize 
	super initialize.
	processes := OrderedCollection new! !

!LoomSimulation methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 9/13/2012 10:11'!
loadCPUWith: floats
	"The sum of numbers should be 1"
	| timeToRunExperiment |
	self assert: [ (floats inject: 0 into: [ :s :e | s + e])  = 1.0 ].
	
	timeToRunExperiment := 10000.
	floats do: [ :f |
		| p |
		p := [ (2000 * f) asInteger 
				timesRepeat: [ (Delay forMilliseconds: (timeToRunExperiment / (2000 * f)) asInteger) wait ].
				Transcript show: 'Process ended!! ', f printString; cr ] forkAt: 41.
		processes add: p.
	].! !

!LoomSimulation methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 9/13/2012 09:12'!
stopProcesses
	processes do: [ :p | p terminate ].
	processes := OrderedCollection new.
 ! !

LoomObject subclass: #LoomSortingThreadStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

LoomSortingThreadStrategy subclass: #LoomConsumptionSortingStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomConsumptionSortingStrategy methodsFor: 'hooks' stamp: 'AlexandreBergel 9/12/2012 15:15'!
sort: loomThreads
	^ loomThreads asSortedCollection: [ :t1 :t2 | t1 consumptionTime > t2 consumptionTime ]! !

LoomObject subclass: #LoomThread
	instanceVariableNames: 'process id tick profiler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Loom'!

!LoomThread class methodsFor: 'message' stamp: 'AlexandreBergel 9/13/2012 10:25'!
wronglyInitializedThreadMessage
	^ 'Uninitialized thread'! !

!LoomThread methodsFor: 'accessing computed' stamp: 'AlexandreBergel 9/12/2012 18:32'!
consumptionTime
	"Just in case that no profile has been done"
	tick = 0 ifTrue: [ ^ 0 ].
	
	^ (((tick / profiler numberOfSamplings) * 100) asInteger / 100) asFloat! !

!LoomThread methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 10:58'!
id
	"Return the id of the thread. If 0, then it means that none has been set"

	^ id! !

!LoomThread methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 10:58'!
id: aNumber
	"Set the id number"
	
	id := aNumber! !

!LoomThread methodsFor: 'public' stamp: 'AlexandreBergel 9/5/2012 11:50'!
increaseTick
	tick := tick + 1! !

!LoomThread methodsFor: 'initialize' stamp: 'AlexandreBergel 9/5/2012 11:50'!
initialize
	super initialize.
	id := 0.
	tick := 0! !

!LoomThread methodsFor: 'testing' stamp: 'AlexandreBergel 9/6/2012 08:45'!
isSuspended
	^ process isSuspended! !

!LoomThread methodsFor: 'testing' stamp: 'AlexandreBergel 9/6/2012 08:45'!
isTerminated
	"Return true if the process is terminated"
	
	^ process isTerminated! !

!LoomThread methodsFor: 'printing' stamp: 'AlexandreBergel 9/13/2012 11:20'!
printOn: stream
	process isNil ifTrue: [ stream nextPutAll: self class wronglyInitializedThreadMessage. ^ self ].
	
	stream nextPutAll: '['.
	stream nextPutAll: (self consumptionTime * 100) asInteger printString.
	stream nextPutAll: '%] ('.
	stream nextPutAll: self priority printString.
	stream nextPutAll: ')'.
	
	self isTerminated 
		ifTrue: [ stream nextPutAll: ' T '].
	
	self printProcessNameOn: stream.

	(process suspendedContext class ==  MethodContext)
		ifTrue: [ stream nextPutAll: process suspendedContext home printString ]
		ifFalse: [ stream nextPutAll: process suspendedContext printString ].

		
	stream nextPutAll: ' ', process hash printString.! !

!LoomThread methodsFor: 'printing' stamp: 'AlexandreBergel 9/13/2012 11:21'!
printProcessNameOn: stream
	"Print the name of the process on stream, if it has one"
	
	"Check if process has a name. If no, then we do nothing"
	(process name isNil or: [ process name first = $ ]) ifTrue: [ ^ self ].
	
	stream nextPutAll: ' ', process name, ' '! !

!LoomThread methodsFor: 'accessing computed' stamp: 'AlexandreBergel 9/13/2012 11:16'!
priority
	"Always return a number."

	"Answer the priority of the receiver."
	process ifNil: [ ^ 1 ].
	
	"Case of Pharo"
	process priority isNil ifTrue: [ ^ 1 ].
		
	^ process priority! !

!LoomThread methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 10:58'!
process
	^ process! !

!LoomThread methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 10:58'!
process: anObject
	process := anObject! !

!LoomThread methodsFor: 'accessing' stamp: 'AlexandreBergel 9/12/2012 09:21'!
profiler
	^ profiler! !

!LoomThread methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 19:29'!
profiler: aProfiler
	profiler := aProfiler! !

!LoomThread methodsFor: 'public' stamp: 'AlexandreBergel 9/12/2012 18:36'!
resetTick
	"Invoked after each sampling"
	
	tick := 0! !

!LoomThread methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2012 11:51'!
tick
	^ tick! !
